searchData={"items":[{"type":"module","title":"Majic","doc":"Robust libmagic integration for Elixir.","ref":"Majic.html"},{"type":"function","title":"Majic.perform/2","doc":"Perform on `path`.\n\nAn option of `server: ServerName`, `pool: PoolName` or `once: true` must be passed.","ref":"Majic.html#perform/2"},{"type":"type","title":"Majic.name/0","doc":"","ref":"Majic.html#t:name/0"},{"type":"type","title":"Majic.option/0","doc":"","ref":"Majic.html#t:option/0"},{"type":"type","title":"Majic.result/0","doc":"","ref":"Majic.html#t:result/0"},{"type":"type","title":"Majic.target/0","doc":"","ref":"Majic.html#t:target/0"},{"type":"module","title":"Majic.Extension","doc":"Helper module to fix extensions. Uses [MIME](https://hexdocs.pm/mime/MIME.html).","ref":"Majic.Extension.html"},{"type":"function","title":"Majic.Extension.fix/3","doc":"Fix `name`'s extension according to `result_or_mime_type`.\n\n```elixir\niex(1)> {:ok, result} = Majic.perform(\"cat.jpeg\", once: true)\n{:ok, %Majic.Result{mime_type: \"image/webp\", ...}}\niex(1)> Majic.Extension.fix(\"cat.jpeg\", result)\n\"cat.webp\"\n```\n\nThe `append: true` option will append the correct extension to the user-provided one, if there's an extension for the\ntype:\n\n```\niex(1)> Majic.Extension.fix(\"cat.jpeg\", result, append: true)\n\"cat.jpeg.webp\"\niex(2)> Majic.Extension.fix(\"Makefile.txt\", \"text/x-makefile\", append: true)\n\"Makefile\"\n```\n\nThe `subtype_as_extension: true` option will use the subtype part of the MIME type as an extension for the ones that\ndon't have any:\n\n```elixir\niex(1)> Majic.Extension.fix(\"Makefile.txt\", \"text/x-makefile\", subtype_as_extension: true)\n\"Makefile.x-makefile\"\niex(1)> Majic.Extension.fix(\"Makefile.txt\", \"text/x-makefile\", subtype_as_extension: true, append: true)\n\"Makefile.txt.x-makefile\"\n```","ref":"Majic.Extension.html#fix/3"},{"type":"type","title":"Majic.Extension.option_append/0","doc":"If an extension is defined for a given MIME type, append it to the previous extension.\n\nIf no extension could be found for the MIME type, and `subtype_as_extension: false`, the returned filename will have no extension.","ref":"Majic.Extension.html#t:option_append/0"},{"type":"type","title":"Majic.Extension.option_subtype_as_extension/0","doc":"If no extension is defined for a given MIME type, use the subtype as its extension.","ref":"Majic.Extension.html#t:option_subtype_as_extension/0"},{"type":"module","title":"Majic.Once","doc":"Contains convenience functions for one-off use.","ref":"Majic.Once.html"},{"type":"function","title":"Majic.Once.perform/3","doc":"Runs a one-shot process without supervision.\n\nUseful in tests, but not recommended for actual applications.","ref":"Majic.Once.html#perform/3"},{"type":"function","title":"Example - Majic.Once.perform/3","doc":"iex(1)> {:ok, result} = Majic.Once.perform(\".\")\n    iex(2)> result\n    %Majic.Result{content: \"directory\", encoding: \"binary\", mime_type: \"inode/directory\"}","ref":"Majic.Once.html#perform/3-example"},{"type":"module","title":"Majic.Plug","doc":"A `Plug` to automatically set the `content_type` of every `Plug.Upload`.\n\nOne of the required option of `pool`, `server` or `once` must be set.\n\nAdditional options:\n* `fix_extension`, default false: enable use of `Majic.Extension`,\n* options for `Majic.Extension`.\n\nTo use a majic pool:\n\n```\nplug Majic.Plug, pool: MyApp.MajicPool\n```\n\nTo use a single majic server:\n\n```\nplug Majic.Plug, server: MyApp.MajicServer\n```\n\nTo start a majic process at each file (not recommended):\n\n```\nplug Majic.Plug, once: true\n```","ref":"Majic.Plug.html"},{"type":"module","title":"Majic.Pool","doc":"Pool of `Majic.Server`","ref":"Majic.Pool.html"},{"type":"function","title":"Majic.Pool.child_spec/1","doc":"","ref":"Majic.Pool.html#child_spec/1"},{"type":"function","title":"Majic.Pool.perform/3","doc":"","ref":"Majic.Pool.html#perform/3"},{"type":"function","title":"Majic.Pool.start_link/1","doc":"","ref":"Majic.Pool.html#start_link/1"},{"type":"type","title":"Majic.Pool.name/0","doc":"","ref":"Majic.Pool.html#t:name/0"},{"type":"type","title":"Majic.Pool.option/0","doc":"","ref":"Majic.Pool.html#t:option/0"},{"type":"module","title":"Majic.Result","doc":"Represents the results obtained from libmagic.\n\nPlease note that this struct is only returned if the underlying check has succeeded.","ref":"Majic.Result.html"},{"type":"type","title":"Majic.Result.t/0","doc":"Represents the result.\n\nContains the MIME type, Encoding and Content fields returned by libmagic, as per the flags:\n\n- MIME Type: `MAGIC_FLAGS_COMMON|MAGIC_MIME_TYPE`\n- Encoding: `MAGIC_FLAGS_COMMON|MAGIC_MIME_ENCODING`\n- Type Name (Content): `MAGIC_FLAGS_COMMON|MAGIC_NONE`","ref":"Majic.Result.html#t:t/0"},{"type":"module","title":"Majic.Server","doc":"Provides access to the underlying libmagic client, which performs file introspection.\n\nThe Server needs to be supervised, since it will terminate if it receives any unexpected error.","ref":"Majic.Server.html"},{"type":"function","title":"Majic.Server.child_spec/1","doc":"Returns the default Child Specification for this Server for use in Supervisors.\n\nYou can override this with `Supervisor.child_spec/2` as required.","ref":"Majic.Server.html#child_spec/1"},{"type":"function","title":"Majic.Server.perform/3","doc":"Determines the type of the file provided.","ref":"Majic.Server.html#perform/3"},{"type":"function","title":"Majic.Server.recycle/3","doc":"Same as `reload/2,3` but with a full restart of the underlying C port.","ref":"Majic.Server.html#recycle/3"},{"type":"function","title":"Majic.Server.reload/3","doc":"Reloads a Server with a new set of databases.","ref":"Majic.Server.html#reload/3"},{"type":"function","title":"Majic.Server.start_link/1","doc":"Starts a new Server.\n\nSee `t:option/0` for further details.","ref":"Majic.Server.html#start_link/1"},{"type":"function","title":"Majic.Server.status/2","doc":"Returns status of the Server.","ref":"Majic.Server.html#status/2"},{"type":"function","title":"Majic.Server.stop/1","doc":"Stops the Server with reason `:normal` and timeout `:infinity`.","ref":"Majic.Server.html#stop/1"},{"type":"function","title":"Majic.Server.stop/3","doc":"Stops the Server with the specified reason and timeout.","ref":"Majic.Server.html#stop/3"},{"type":"type","title":"Majic.Server.start_option/0","doc":"Represents values accepted as startup options, which can be passed to `start_link/1`.\n\n- `:name`: If present, this will be the registered name for the underlying process.\n  Note that `:gen_statem` requires `{:local, name}`, but given widespread GenServer convention,\n  atoms are accepted and will be converted to `{:local, name}`.\n\n- `:startup_timeout`: Specifies how long the Server waits for the C program to initialise.\n  However, if the underlying C program exits, then the process exits immediately.\n\n  Can be set to `:infinity`.\n\n- `:process_timeout`: Specifies how long the Server waits for each request to complete.\n\n  Can be set to `:infinity`.\n\n  Please note that, if you have chosen a custom timeout value, you should also pass it when\n  using `Majic.Server.perform/3`.\n\n- `:recycle_threshold`: Specifies the number of requests processed before the underlying C\n  program is recycled.\n\n  Can be set to `:infinity` if you do not wish for the program to be recycled.\n\n- `:database_patterns`: Specifies what magic databases to load; you can specify a list of files, or of\n  Path Patterns (see `Path.wildcard/2`) or `:default` to instruct the C program to load the\n  appropriate databases.\n\n  For example, if you have had to add custom magics, then you can set this value to:\n\n      [:default, \"path/to/my/magic\"]","ref":"Majic.Server.html#t:start_option/0"},{"type":"type","title":"Majic.Server.state/0","doc":"Current state of the Server:\n\n- `:pending`: This is the initial state; the Server will attempt to start the underlying Port\n  and the libmagic client, then automatically transition to either Available or Crashed.\n\n- `:available`: This is the default state. In this state the Server is able to accept requests\n  and they will be replied in the same order.\n\n- `:processing`: This is the state the Server will be in if it is processing requests. In this\n  state, further requests can still be lodged and they will be processed when the Server is\n  available again.\n\n  For proper concurrency, use a process pool like Poolboy, Sbroker, etc.\n\n- `:recycling`: This is the state the Server will be in, if its underlying C program needs to be\n  recycled. This state is triggered whenever the cycle count reaches the defined value as per\n  `:recycle_threshold`.\n\n  In this state, the Server is able to accept requests, but they will not be processed until the\n  underlying C server program has been started again.","ref":"Majic.Server.html#t:state/0"},{"type":"type","title":"Majic.Server.t/0","doc":"Represents the reference to the underlying server, as returned by `:gen_statem`.","ref":"Majic.Server.html#t:t/0"},{"type":"module","title":"Majic.Server.Status","doc":"Represents Status of the underlying Server.","ref":"Majic.Server.Status.html"},{"type":"type","title":"Majic.Server.Status.t/0","doc":"Represnets Staus of the Server.\n\n- `:state`: Represents the current state of the Server\n\n- `:cycles`: Represents the number of cycles the Server has run; note that this resets if\n  recycling is enabled.","ref":"Majic.Server.Status.html#t:t/0"},{"type":"exception","title":"Majic.PlugError","doc":"","ref":"Majic.PlugError.html"},{"type":"extras","title":"Majic","doc":"# Majic\n\n**Majic** provides a robust integration of [libmagic](http://man7.org/linux/man-pages/man3/libmagic.3.html) for Elixir.\n\nWith this library, you can start an one-off process to run a single check, or run the process as a daemon if you expect to run\nmany checks.\n\nIt is a friendly fork of [gen_magic](https://github.com/evadne/gen_magic) featuring a (arguably) more robust C-code\nusing erl_interface, built in pooling, unified/clean API, and an optional Plug.\n\nThis package is regulary tested on multiple platforms (Debian, macOS, Fedora, Alpine, FreeBSD) to ensure it'll work fine\nin any environment.","ref":"readme.html"},{"type":"extras","title":"Installation - Majic","doc":"The package can be installed by adding `majic` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:majic, \"~> 1.0\"}\n  ]\nend\n```\n\nYou must also have [libmagic](http://man7.org/linux/man-pages/man3/libmagic.3.html) installed locally with headers, alongside common compilation tools (i.e. build-essential). These can be acquired by apt-get, yum, brew, etc.\n\nCompilation of the underlying C program is automatic and handled by [elixir_make](https://github.com/elixir-lang/elixir_make).","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Majic","doc":"Depending on the use case, you may utilise a single (one-off) Majic process without reusing it as a daemon, or utilise a connection pool (such as Poolboy) in your application to run multiple persistent Majic processes.\n\nTo use Majic directly, you can use `Majic.Once.perform/1`:\n\n```elixir\niex(1)> Majic.perform(\".\", once: true)\n{:ok,\n %Majic.Result{\n   content: \"directory\",\n   encoding: \"binary\",\n   mime_type: \"inode/directory\"\n }}\n```\n\nTo use the Majic server as a daemon, you can start it first, keep a reference, then feed messages to it as you require:\n\n```elixir\n{:ok, pid} = Majic.Server.start_link([])\n{:ok, result} = Majic.perform(path, server: pid)\n```\n\nSee `Majic.Server.start_link/1` and `t:Majic.Server.option/0` for more information on startup parameters.\n\nSee `Majic.Result` for details on the result provided.","ref":"readme.html#usage"},{"type":"extras","title":"Configuration - Majic","doc":"When using `Majic.Server.start_link/1` to start a persistent server, or `Majic.Helpers.perform_once/2` to run an ad-hoc request, you can override specific options to suit your use case.\n\n| Name | Default | Description |\n| - | - | - |\n| `:startup_timeout` | 1000 | Number of milliseconds to wait for client startup |\n| `:process_timeout` | 30000 | Number of milliseconds to process each request |\n| `:recycle_threshold` | 10 | Number of cycles before the C process is replaced |\n| `:database_patterns` | `[:default]` | Databases to load |\n\nSee `t:Majic.Server.option/0` for details.\n\n__Note__ `:recycle_thresold` is only useful if you are using a libmagic `<5.29`, where it was susceptible to memleaks\n([details](https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=840754)]). In future versions of `majic` this option could\nbe ignored.","ref":"readme.html#configuration"},{"type":"extras","title":"Reloading / Altering databases - Majic","doc":"If you want `majic` to reload its database(s), run `Majic.Server.reload(ref)`.\n\nIf you want to add or remove databases to a running server, you would have to run `Majic.Server.reload(ref, databases)`\nwhere databases being the same argument as `database_patterns` on start. `Majic` does not support adding/removing\ndatabases at runtime without a port reload.","ref":"readme.html#reloading-altering-databases"},{"type":"extras","title":"Use Cases - Majic","doc":"#### Ad-Hoc Requests\n\nFor ad-hoc requests, you can use the helper method `Majic.Once.perform_once/2`:\n\n```elixir\niex(1)> Majic.perform(Path.join(File.cwd!(), \"Makefile\"), once: true)\n{:ok,\n %Majic.Result{\n   content: \"makefile script, ASCII text\",\n   encoding: \"us-ascii\",\n   mime_type: \"text/x-makefile\"\n}}\n```\n\n#### Supervised Requests\n\nThe Server should be run under a supervisor which provides resiliency.\n\nHere we run it under a supervisor in an application:\n\n```elixir\nchildren =\n  [\n    # ...\n    {Majic.Server, [name: YourApp.Majic]}\n  ]\n\nopts = [strategy: :one_for_one, name: YourApp.Supervisor]\nSupervisor.start_link(children, opts)\n```\n\nNow we can ask it to inspect a file:\n\n```elixir\niex(2)> Majic.perform(Path.expand(\"~/.bash_history\"), server: YourApp.Majic)\n{:ok, %Majic.Result{mime_type: \"text/plain\", encoding: \"us-ascii\", content: \"ASCII text\"}}\n```\n\nNote that in this case we have opted to use a named process.\n\n#### Pool\n\nFor concurrency *and* resiliency, you may start the `Majic.Pool`. By default, it will start a `Majic.Server`\nworker per online scheduler:\n\nYou can add a pool in your application supervisor by adding it as a child:\n\n```elixir\nchildren =\n  [\n    # ...\n    {Majic.Pool, [name: YourApp.MajicPool, pool_size: 2]}\n  ]\n\nopts = [strategy: :one_for_one, name: YourApp.Supervisor]\nSupervisor.start_link(children, opts)\n```\n\nAnd then you can use it with `Majic.perform/2` with `pool: YourApp.MajicPool` option:\n\n```elixir\niex(1)> Majic.perform(Path.expand(\"~/.bash_history\"), pool: YourApp.MajicPool)\n{:ok, %Majic.Result{mime_type: \"text/plain\", encoding: \"us-ascii\", content: \"ASCII text\"}}\n```\n#### Fixing extensions\n\nYou may also want to fix the user-provided filename according to its detected MIME type. To do this, you can use `Majic.Extension.fix/3`:\n\n```elixir\niex(1)> {:ok, result} = Majic.perform(\"cat.jpeg\", once: true)\n{:ok, %Majic.Result{mime_type: \"image/webp\", ...}}\niex(1)> Majic.Extension.fix(\"cat.jpeg\", result)\n\"cat.webp\"\n```\n\n#### Use with Plug.Upload\n\nIf you use Plug or Phoenix, you may want to automatically verify the content type of every `Plug.Upload`. The\n`Majic.Plug` is there for this.\n\nEnable it by using `plug Majic.Plug, pool: YourApp.MajicPool` in your pipeline or controller. Then, every `Plug.Upload`\nin `conn.params` and `conn.body_params` is now verified. The filename is also altered with an extension matching its\ncontent-type, using `Majic.Extension`.","ref":"readme.html#use-cases"},{"type":"extras","title":"Notes - Majic","doc":"","ref":"readme.html#notes"},{"type":"extras","title":"Soak Test - Majic","doc":"Run an endless cycle to prove that the program is resilient:\n\n```bash\nfind /usr/share/ -name *png | xargs mix run test/soak.exs\nfind . -name *ex | xargs mix run test/soak.exs\n```","ref":"readme.html#soak-test"},{"type":"extras","title":"Acknowledgements - Majic","doc":"During design and prototype development of this library, the Author has drawn inspiration from the following individuals, and therefore\nthanks all contributors for their generosity:\n\n- [Evadne Wu](https://github.com/evadne)\n  - Original [gen_magic](https://github.com/evadne/gen_magic) author.\n- [James Every](https://github.com/devstopfix)\n  - Enhanced Elixir Wrapper (based on GenServer)\n  - Initial Hex packaging (v.0.22)\n  - Soak Testing\n- Matthias and Ced for helping the author with C oddities\n- [Hecate](https://github.com/Kleidukos) for laughing at aforementionned oddities\n- majic for giving inspiration for the lib name (magic, majic, get it? hahaha..)","ref":"readme.html#acknowledgements"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog][1], and this project adheres to [Semantic Versioning][2].\n\n[1]: https://keepachangelog.com/en/1.0.0/\n[2]: https://semver.org/spec/v2.0.0.html","ref":"changelog.html"},{"type":"extras","title":"majic [1.1.1] - Changelog","doc":"- Fixed build artifact mistakenly included in the Hex.pm package","ref":"changelog.html#majic-1-1-1"},{"type":"extras","title":"majic [1.1.0] - Changelog","doc":"- Updated dependencies\n- `-Werror` removed from `Makefile`\n- `Majic.Extension.fix/3` will append an extension when requested if none exist","ref":"changelog.html#majic-1-1-0"},{"type":"extras","title":"majic [1.0.0] - Changelog","doc":"","ref":"changelog.html#majic-1-0-0"},{"type":"extras","title":"Added - Changelog","doc":"- Forked gen_magic.\n- Pool: `Majic.Pool`, using [nimble_pool](https://hex.pm/packages/nimble_pool).\n- Plug: `Majic.Plug`.\n- Unified API: `Majic.perform/1,2,3`.\n- Builds an up-to-date and patched magic database.\n- `Majic.compile/2`\n- `Majic.Server.reload/2,3`\n- `Majic.Server.recycle/2,3`","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Improved C port, now using erl_interface\n- Builds on Musl\n- Better error and timeout handling\n- Bytes support: `Majic.Server.perform(ref, {:bytes, <<>>})`\n- Renamed `priv/apprentice` to `priv/libmagic_port` to be more obvious in `ps`\n- Renamed `Majic.Helpers.perform_once` to `Majic.Once.perform`","ref":"changelog.html#changed"},{"type":"extras","title":"gen_majic [1.0] - Changelog","doc":"","ref":"changelog.html#gen_majic-1-0"},{"type":"extras","title":"Added - Changelog","doc":"- Added support for process recycling (evadne).\n- Added documentation (evadne).","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Replaced GenServer with `:gen_statem` (evadne).\n  - Changed API; added support for customisation.\n\n- Refined tests and other aspects of the library (evadne).\n\n## [0.20.83]","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Soak testing script (devstopfix)","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Replaced Erlexec usage with Port (devstopfix)","ref":"changelog.html#changed"},{"type":"extras","title":"0.0.1 - Changelog","doc":"","ref":"changelog.html#0-0-1"},{"type":"extras","title":"Added - Changelog","doc":"- Initial Elixir wrapper with Erlexec (evadne)\n- Intiial C program (evadne)\n\n[unreleased]: https://github.com/evadne/gen_magic/compare/develop\n[0.20.83]: https://github.com/devstopfix/gen_magic/commit/7e27fd094cb462d26ba54fde0205a5be313d12da","ref":"changelog.html#added"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}